%                       ROBOT DELTA'S PROJECT 2012
%                    UNIVERSIDAD AUTNOMA DE BUCARAMANGA
%                        INGENIERA MECATRNICA

%%         STATIC SIMULATION OF A DELTA ROBOT  By DANIEL - FABIAN
%                    Units in millimeters
%% VALUES OF INICIALIZATION
          clear all
          close all
          clc
l1 = 225;
l2 = 364;
l3 = 415;
dT1 = 480;
lp = 82;
pt = 20;
bar = 400;
%% STATIC GEOMETRY
O = eye(4);
ang1 = 0;
ang2 = 120;
ang3 = 240;
ang4 = 115;
rz1 = [cosd(ang1) -sind(ang1) 0 0;sind(ang1) cosd(ang1) 0 0; 0 0 1 0;0 0 0 1];
rz2 = [cosd(ang2) -sind(ang2) 0 0;sind(ang2) cosd(ang2) 0 0; 0 0 1 0;0 0 0 1];
rz3 = [cosd(ang3) -sind(ang3) 0 0;sind(ang3) cosd(ang3) 0 0; 0 0 1 0;0 0 0 1];
T1 = [1 0 0 0; 0 1 0 0; 0 0 1 dT1;0 0 0 1];
T2 = [1 0 0 l1; 0 1 0 0; 0 0 1 0;0 0 0 1];
ry1 = [cosd(ang4) 0 sind(ang4) 0;0 1 0 0;-sind(ang4) 0 cosd(ang4) 0;0 0 0 1];
T3 = [1 0 0 l3; 0 1 0 0; 0 0 1 0;0 0 0 1];
Rm = lp/(sqrt(3));
% OTHERS COODINATES
a1 = O*T1;
a2 = a1*rz1*T2;
a3 = a1*rz2*T2;
a4 = a1*rz3*T2;
b1 = a2*ry1*T3;
b2 = a3*ry1*T3;
b3 = a4*ry1*T3;
%% KEY POINTS
one = a1(1:3,4);
two = a2(1:3,4);
three = a3(1:3,4);
four = a4(1:3,4);
five = b1(1:3,4);
six = b2(1:3,4);
seven = b3(1:3,4);
eight = [0;0;b1(3,4)];
%% SHAPE GEOMETRY
Tzn = [1 0 0 0; 0 1 0 0; 0 0 1 pt;0 0 0 1];
PHx = cosd(ang1)*Rm;
PHy = sind(ang1)*Rm;
THxy = [1 0 0 PHx; 0 1 0 PHy; 0 0 1 0;0 0 0 1];
PIx = cosd(ang2)*Rm;
PIy = sind(ang2)*Rm;
TIxy = [1 0 0 PIx; 0 1 0 PIy; 0 0 1 0;0 0 0 1];
PJx = cosd(ang3)*Rm;
PJy = sind(ang3)*Rm;
TJxy = [1 0 0 PJx; 0 1 0 PJy; 0 0 1 0;0 0 0 1];
%% HANDLER ORIENTATION
     rx = 0;
     ry = 0;
RPx = [1 0 0 0;0 cosd(rx) -sind(rx) 0;0 sind(rx) cosd(rx) 0;0 0 0 1];
RPy = [cosd(ry) 0 sind(ry) 0;0 1 0 0;-sind(ry) 0 cosd(ry) 0;0 0 0 1];
     tpx = 100;
     tpy = 0;
     tpz = -100;
TPxy = [1 0 0 tpx; 0 1 0 tpy; 0 0 1 tpz;0 0 0 1];
P = O*TPxy*RPx*RPy;
PP = P((1:3),4);
Q = P*Tzn;
PQ = Q((1:3),4);
%% SHAPE POINTS 3D
H = Q*THxy;
PH = H((1:3),4);
I = Q*TIxy;
PI = I((1:3),4);
J = Q*TJxy;
PJ = J((1:3),4);
%% POSITION VECTORS
nBE = norm(two-five);
uBE = (two-five)./nBE;
nCF = norm(three-six);
uCF = (three-six)./nCF;
nDG = norm(four-seven);
uDG = (four-seven)./nDG;
%% ITERATION PART
N = 4000;
adv = linspace(0,l3,N);

counter = 1;
point = 1;
PW = [PH,PI,PJ];
uW = [uBE,uCF,uDG];
number = [two,three,four];
while point <= 3
    for i=1:N
        AV = uW(:,point)*adv(i);
        POS = number(:,point)-AV;
        SIGMA = norm(POS-PW(:,point));
        ERROR = abs(SIGMA-bar);
        if ERROR <= 0.1 
            PS(1,point) = POS(1,1);
            PS(2,point) = POS(2,1);
            PS(3,point) = POS(3,1);
            VL(point) = SIGMA;
            vrW(point) = norm(eight-PW(:,point));
            counter = 1;
            break;
        end
        counter = counter + 1;
    end
    
    if counter >= N || vrW(point) <= Rm
        disp(' ');disp(['LA POSICION ',num2str(point),': NO EXISTE O MECANICAMENTE ES IMPOSIBLE']);disp(' ');
        break;
    end
    point = point + 1;
end


%% GRAPHICS
% MAIN COORDINATE SYSTEM
frame(O,'r',50);grid off; axis([-250 250 -250 250 0 500]);view([60 40 60]);hold on;
% REPRESENTATIVE POINTS
text(one(1,1)+20,one(2,1),one(3,1),'A','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(two(1,1)+20,two(2,1)-20,two(3,1),'B','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(three(1,1)+20,three(2,1)+20,three(3,1),'C','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(four(1,1)+20,four(2,1)-20,four(3,1),'D','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(five(1,1)+20,five(2,1)-20,five(3,1),'E','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(six(1,1)+20,six(2,1)+20,six(3,1),'F','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
text(seven(1,1)-20,seven(2,1)-20,seven(3,1),'G','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);

plot3([one(1,1) two(1,1)],[one(2,1) two(2,1)],[one(3,1) two(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([one(1,1) three(1,1)],[one(2,1) three(2,1)],[one(3,1) three(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([one(1,1) four(1,1)],[one(2,1) four(2,1)],[one(3,1) four(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([two(1,1) five(1,1)],[two(2,1) five(2,1)],[two(3,1) five(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([three(1,1) six(1,1)],[three(2,1) six(2,1)],[three(3,1) six(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([four(1,1) seven(1,1)],[four(2,1) seven(2,1)],[four(3,1) seven(3,1)],'Color',[.53 .54 .56],'linewidth',2);
plot3([one(1,1) eight(1,1)],[one(2,1) eight(2,1)],[one(3,1) eight(3,1)],'Color',[.53 .54 .56],'linewidth',2);

sx=[five(1,1) six(1,1) seven(1,1)]';
sy=[five(2,1) six(2,1) seven(2,1)]';
sz=[five(3,1) six(3,1) seven(3,1)]';
patch(sx,sy,sz,[.53 .54 .56]);
% MAIN SHAPE AND HANDLER TOOL
SX=[PH(1,1) PI(1,1) PJ(1,1)]';
SY=[PH(2,1) PI(2,1) PJ(2,1)]';
SZ=[PH(3,1) PI(3,1) PJ(3,1)]';
patch(SX,SY,SZ,[.65 .8 .2]);
plot3([PP(1,1) PQ(1,1)],[PP(2,1) PQ(2,1)],[PP(3,1) PQ(3,1)],'Color',[.0 .0 .8],'linewidth',2);

% TEST POSITION
plot3(PS(1,1),PS(2,1),PS(3,1),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8);
plot3(PS(1,2),PS(2,2),PS(3,2),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
plot3(PS(1,3),PS(2,3),PS(3,3),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)

plot3([PS(1,1) PH(1,1)],[PS(2,1) PH(2,1)],[PS(3,1) PH(3,1)],'Color',[.8 .6 .2],'linewidth',2);
plot3([PS(1,2) PI(1,1)],[PS(2,2) PI(2,1)],[PS(3,2) PI(3,1)],'Color',[.8 .6 .2],'linewidth',2);
plot3([PS(1,3) PJ(1,1)],[PS(2,3) PJ(2,1)],[PS(3,3) PJ(3,1)],'Color',[.8 .6 .2],'linewidth',2);

axis square





















