%                       ROBOT DELTA'S PROJECT 2012
%                    UNIVERSIDAD AUTNOMA DE BUCARAMANGA
%                        INGENIERA MECATRNICA

%%         TEST SIMULATION OF A DELTA ROBOT  By DANIEL - FABIAN
%                    Units in millimeters
%% VALUES OF INICIALIZATION
          clear all
          close all
          clc
l1 = 225;
l2 = 364;
l3 = 415;
dT1 = 480;
lp = 82;
pt = 20;
bar = 400;
%% USER DATA
Pti = [30 ; 30 ; -100];       % starting point of the handler in use
operrx = 30;                  % angle of rotation in the x-axis
operry = 0;                   % angle of rotation in the y-axis
%% STATIC GEOMETRY
O = eye(4);
ang1 = 0;
ang2 = 120;
ang3 = 240;
ang4 = 115;
rz1 = [cosd(ang1) -sind(ang1) 0 0;sind(ang1) cosd(ang1) 0 0; 0 0 1 0;0 0 0 1];
rz2 = [cosd(ang2) -sind(ang2) 0 0;sind(ang2) cosd(ang2) 0 0; 0 0 1 0;0 0 0 1];
rz3 = [cosd(ang3) -sind(ang3) 0 0;sind(ang3) cosd(ang3) 0 0; 0 0 1 0;0 0 0 1];
T1 = [1 0 0 0; 0 1 0 0; 0 0 1 dT1;0 0 0 1];
T2 = [1 0 0 l1; 0 1 0 0; 0 0 1 0;0 0 0 1];
ry1 = [cosd(ang4) 0 sind(ang4) 0;0 1 0 0;-sind(ang4) 0 cosd(ang4) 0;0 0 0 1];
T3 = [1 0 0 l3; 0 1 0 0; 0 0 1 0;0 0 0 1];
Rm = lp/(sqrt(3));
% OTHERS COODINATES
a1 = O*T1;
a2 = a1*rz1*T2;
a3 = a1*rz2*T2;
a4 = a1*rz3*T2;
b1 = a2*ry1*T3;
b2 = a3*ry1*T3;
b3 = a4*ry1*T3;
%% KEY POINTS
one = a1(1:3,4);
two = a2(1:3,4);
three = a3(1:3,4);
four = a4(1:3,4);
five = b1(1:3,4);
six = b2(1:3,4);
seven = b3(1:3,4);
eight = [0;0;b1(3,4)];
%% SHAPE GEOMETRY
Tzn = [1 0 0 0; 0 1 0 0; 0 0 1 pt;0 0 0 1];
PHx = cosd(ang1)*Rm;
PHy = sind(ang1)*Rm;
THxy = [1 0 0 PHx; 0 1 0 PHy; 0 0 1 0;0 0 0 1];
PIx = cosd(ang2)*Rm;
PIy = sind(ang2)*Rm;
TIxy = [1 0 0 PIx; 0 1 0 PIy; 0 0 1 0;0 0 0 1];
PJx = cosd(ang3)*Rm;
PJy = sind(ang3)*Rm;
TJxy = [1 0 0 PJx; 0 1 0 PJy; 0 0 1 0;0 0 0 1];
%% POSITION VECTORS
nBE = norm(two-five);
uBE = (two-five)./nBE;
nCF = norm(three-six);
uCF = (three-six)./nCF;
nDG = norm(four-seven);
uDG = (four-seven)./nDG;
%% CIRCLE PATH
cx = Pti(1);cy = Pti(2);cz = Pti(3);          % center of the circle
ratio = 50;                                  % ratio
n = 300;                                      % numbers of points
anglerut = linspace(0,2*pi,n);xr = cos(anglerut).*ratio;yr = sin(anglerut).*ratio;
axc = 0;
byc = 0;
czc = 0;
roxr = [1 0 0 0;0 cosd(axc) -sind(axc) 0;0 sind(axc) cosd(axc) 0;0 0 0 1]; 
royr = [cosd(byc) 0 sind(byc) 0;0 1 0 0;-sind(byc) 0 cosd(byc) 0;0 0 0 1]; 
rozr = [cosd(czc) -sind(czc) 0 0;sind(czc) cosd(czc) 0 0; 0 0 1 0;0 0 0 1]; 
tcentro = [1 0 0 cx; 0 1 0 cy; 0 0 1 cz;0 0 0 1];
for j = 1:1:n
    txr = xr(j);
    tyr = yr(j);
    tzr = 0;
    tras = [1 0 0 txr; 0 1 0 tyr; 0 0 1 tzr;0 0 0 1];
    AUX = O*tcentro*roxr*royr*rozr*tras;
    XRC(j) = AUX(1:1,4:4);
    YRC(j) = AUX(2:2,4:4);
    ZRC(j) = AUX(3:3,4:4);
end
%% PATH ANALASYS
opertpx = 0;
opertpy = 0;
opertpz = 0;
disp = abs(Pti(1)) + abs(Pti(2)) + abs(Pti(3));
mpfpi = norm([XRC(n);YRC(n);ZRC(n)] - Pti);
upfpi = ([XRC(n);YRC(n);ZRC(n)] - [0;0;0])./mpfpi;
divpfpi = 2*floor(mpfpi);
incpfpi = linspace(0,mpfpi,divpfpi);
for k = 1: divpfpi
    vpfpix(k) = XRC(n) - upfpi(1)*incpfpi(k);
    vpfpiy(k) = YRC(n) - upfpi(2)*incpfpi(k);
    vpfpiz(k) = ZRC(n) - upfpi(3)*incpfpi(k);
end
Frames = disp + ratio + n + divpfpi;
%% SET VIDEO
nroFotogramas = Frames; 
comp = figure(1);
taman =[0,50,600,600];
set(comp,'Position',taman);
set(comp,'NextPlot','replacechildren');
pelicula = moviein(nroFotogramas, comp, taman);
%% ENABLE FOR GENERATE VIDEO .AVI
% set(comp, 'DoubleBuffer', 'On');
% mor = avifile('VIDEO');
%% LOOP FOR SIMULATION MOTION
for i = 1: Frames
     % MOVEMENTS
     if i <= abs(Pti(1))
         if Pti(1) < 0
             opertpx = -i;
         else
             opertpx = i;
         end
     elseif i <= abs(Pti(1)) + abs(Pti(2))
         if Pti(2) < 0
             opertpy = -(i - abs(Pti(1)));
         else
             opertpy = (i - abs(Pti(1)));
         end
     elseif i <= disp
         if Pti(3) < 0
             opertpz = -(i - (abs(Pti(1)) + abs(Pti(2))));
         else
             opertpz = (i - (abs(Pti(1)) + abs(Pti(2))));
         end
     elseif i <= disp + ratio
         opertpx = Pti(1) + (i - disp);
     elseif i <= disp + ratio + n
         opertpx = XRC(i - (disp+ratio));
         opertpy = YRC(i - (disp+ratio));
         opertpz = ZRC(i - (disp+ratio));
     else
         opertpx = vpfpix(i-(disp + ratio + n));
         opertpy = vpfpiy(i-(disp + ratio + n));
         opertpz = vpfpiz(i-(disp + ratio + n));
     end
    
     %% HANDLER ORIENTATION
     rx = operrx;
     ry = operry;
     tpx = opertpx;
     tpy = opertpy;
     tpz = opertpz;
     % BACK UP POSITION VECTOR
     Buvx(i) = opertpx;
     Buvy(i) = opertpy;
     Buvz(i) = opertpz;
     
     RPx = [1 0 0 0;0 cosd(rx) -sind(rx) 0;0 sind(rx) cosd(rx) 0;0 0 0 1];
     RPy = [cosd(ry) 0 sind(ry) 0;0 1 0 0;-sind(ry) 0 cosd(ry) 0;0 0 0 1];
     TPxy = [1 0 0 tpx; 0 1 0 tpy; 0 0 1 tpz;0 0 0 1];
     P = O*TPxy*RPx*RPy;
     PP = P((1:3),4);
     Q = P*Tzn;
     PQ = Q((1:3),4);
     %% SHAPE POINTS 3D
     H = Q*THxy;
     PH = H((1:3),4);
     I = Q*TIxy;
     PI = I((1:3),4);
     J = Q*TJxy;
     PJ = J((1:3),4);
     %% ITERATION PART
     N = 4000;
     adv = linspace(0,l3,N);
     
     counter = 1;
     point = 1;
     PW = [PH,PI,PJ];
     uW = [uBE,uCF,uDG];
     number = [two,three,four];
     while point <= 3
         for j=1:N
             AV = uW(:,point)*adv(j);
             POS = number(:,point)-AV;
             SIGMA = norm(POS-PW(:,point));
             ERROR = abs(SIGMA-bar);
             if ERROR <= 0.1
                 PS(1,point) = POS(1,1);
                 PS(2,point) = POS(2,1);
                 PS(3,point) = POS(3,1);
                 VL(point) = SIGMA;
                 vrW(point) = norm(eight-PW(:,point));
                 counter = 1;
                 break;
             end
             counter = counter + 1;
         end
         
         if counter >= N || vrW(point) <= Rm
             disp(' ');disp(['LA POSICION ',num2str(point),': NO EXISTE O MECANICAMENTE ES IMPOSIBLE']);disp(' ');
             break;
         end
         point = point + 1;
     end
     % BACK UP OF SLIDER POSITION
     SP1(:,i) = PS(:,1);
     SP2(:,i) = PS(:,2);
     SP3(:,i) = PS(:,3);
     
     %% GRAPHICS
     % MAIN COORDINATE SYSTEM
     clf
     % frame(O,'r',50);
     grid off; axis([-250 250 -250 250 0 500]);view([60 40 80]);hold on;
     % REPRESENTATIVE POINTS
     text(one(1,1)+20,one(2,1),one(3,1),'A','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(two(1,1)+20,two(2,1)-20,two(3,1),'B','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(three(1,1)+20,three(2,1)+20,three(3,1),'C','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(four(1,1)+20,four(2,1)-20,four(3,1),'D','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(five(1,1)+20,five(2,1)-20,five(3,1),'E','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(six(1,1)+20,six(2,1)+20,six(3,1),'F','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(seven(1,1)-20,seven(2,1)-20,seven(3,1),'G','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     
     plot3([one(1,1) two(1,1)],[one(2,1) two(2,1)],[one(3,1) two(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) three(1,1)],[one(2,1) three(2,1)],[one(3,1) three(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) four(1,1)],[one(2,1) four(2,1)],[one(3,1) four(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([two(1,1) five(1,1)],[two(2,1) five(2,1)],[two(3,1) five(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([three(1,1) six(1,1)],[three(2,1) six(2,1)],[three(3,1) six(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([four(1,1) seven(1,1)],[four(2,1) seven(2,1)],[four(3,1) seven(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) eight(1,1)],[one(2,1) eight(2,1)],[one(3,1) eight(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     
     sx=[five(1,1) six(1,1) seven(1,1)]';
     sy=[five(2,1) six(2,1) seven(2,1)]';
     sz=[five(3,1) six(3,1) seven(3,1)]';
     patch(sx,sy,sz,[.53 .54 .56]);
     % MAIN SHAPE AND HANDLER TOOL
     SX=[PH(1,1) PI(1,1) PJ(1,1)]';
     SY=[PH(2,1) PI(2,1) PJ(2,1)]';
     SZ=[PH(3,1) PI(3,1) PJ(3,1)]';
     patch(SX,SY,SZ,[.65 .8 .2]);
     plot3([PP(1,1) PQ(1,1)],[PP(2,1) PQ(2,1)],[PP(3,1) PQ(3,1)],'Color',[.0 .0 .8],'linewidth',2);
     
     % TEST POSITION
     plot3(PS(1,1),PS(2,1),PS(3,1),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8);
     plot3(PS(1,2),PS(2,2),PS(3,2),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
     plot3(PS(1,3),PS(2,3),PS(3,3),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
     
     plot3([PS(1,1) PH(1,1)],[PS(2,1) PH(2,1)],[PS(3,1) PH(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     plot3([PS(1,2) PI(1,1)],[PS(2,2) PI(2,1)],[PS(3,2) PI(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     plot3([PS(1,3) PJ(1,1)],[PS(2,3) PJ(2,1)],[PS(3,3) PJ(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     % PATH OF THE TRAJECTORY
     for k = 1:i
         plot3(Buvx(k),Buvy(k),Buvz(k),'.b','MarkerSize',4);
     end
     axis square
     pause (0.01)
%      pelicula(:, Frames) = getframe(comp, taman);
%      MOVV = getframe(comp);
%      mor = addframe(mor,MOVV);
end
%      mor = close(mor);