%                       ROBOT DELTA'S PROJECT 2012
%                    UNIVERSIDAD AUTNOMA DE BUCARAMANGA
%                        INGENIERA MECATRNICA

%%         MOVEMENT SIMULATION OF A DELTA ROBOT  By DANIEL - FABIAN
%                    Units in millimeters
%% VALUES OF INICIALIZATION
          clear all
          close all
          clc
l1 = 225;
l2 = 364;
l3 = 415;
dT1 = 480;
lp = 82;
pt = 20;
bar = 400;
%% USER DATA
accel = 2.5;       % acceleration [mm/s^2]
pad = 0.3;         % percent of acceleration
Pti = [0;0;0];     % initial point of the handler
operrx = 30;       % angle of rotation in the x-axis
operry = 0;        % angle of rotation in the y-axis
%% STATIC GEOMETRY
O = eye(4);
ang1 = 0;
ang2 = 120;
ang3 = 240;
ang4 = 115;
rz1 = [cosd(ang1) -sind(ang1) 0 0;sind(ang1) cosd(ang1) 0 0; 0 0 1 0;0 0 0 1];
rz2 = [cosd(ang2) -sind(ang2) 0 0;sind(ang2) cosd(ang2) 0 0; 0 0 1 0;0 0 0 1];
rz3 = [cosd(ang3) -sind(ang3) 0 0;sind(ang3) cosd(ang3) 0 0; 0 0 1 0;0 0 0 1];
T1 = [1 0 0 0; 0 1 0 0; 0 0 1 dT1;0 0 0 1];
T2 = [1 0 0 l1; 0 1 0 0; 0 0 1 0;0 0 0 1];
ry1 = [cosd(ang4) 0 sind(ang4) 0;0 1 0 0;-sind(ang4) 0 cosd(ang4) 0;0 0 0 1];
T3 = [1 0 0 l3; 0 1 0 0; 0 0 1 0;0 0 0 1];
Rm = lp/(sqrt(3));
% OTHERS COODINATES
a1 = O*T1;
a2 = a1*rz1*T2;
a3 = a1*rz2*T2;
a4 = a1*rz3*T2;
b1 = a2*ry1*T3;
b2 = a3*ry1*T3;
b3 = a4*ry1*T3;
%% KEY POINTS
one = a1(1:3,4);
two = a2(1:3,4);
three = a3(1:3,4);
four = a4(1:3,4);
five = b1(1:3,4);
six = b2(1:3,4);
seven = b3(1:3,4);
eight = [0;0;b1(3,4)];
%% SHAPE GEOMETRY
Tzn = [1 0 0 0; 0 1 0 0; 0 0 1 pt;0 0 0 1];
PHx = cosd(ang1)*Rm;
PHy = sind(ang1)*Rm;
THxy = [1 0 0 PHx; 0 1 0 PHy; 0 0 1 0;0 0 0 1];
PIx = cosd(ang2)*Rm;
PIy = sind(ang2)*Rm;
TIxy = [1 0 0 PIx; 0 1 0 PIy; 0 0 1 0;0 0 0 1];
PJx = cosd(ang3)*Rm;
PJy = sind(ang3)*Rm;
TJxy = [1 0 0 PJx; 0 1 0 PJy; 0 0 1 0;0 0 0 1];
%% POSITION VECTORS
nBE = norm(two-five);
uBE = (two-five)./nBE;
nCF = norm(three-six);
uCF = (three-six)./nCF;
nDG = norm(four-seven);
uDG = (four-seven)./nDG;
%% PATH ANALASYS
% PATH POINTS
NP = 4;
P1 = [0;0;-50];
P2 = [0;-100;-50];
P3 = [0;-100;-100];
P4 = [0;0;0];
% NORM AND UNITARY VECTORS
mp0p1 = norm(Pti-P1);
up0p1 = (Pti-P1)./mp0p1;
mp1p2 = norm(P1-P2);
up1p2 = (P1-P2)./mp1p2;
mp2p3 = norm(P2-P3);
up2p3 = (P2-P3)./mp2p3;
mp3p4 = norm(P3-P4);
up3p4 = (P3-P4)./mp3p4;
ESP = 600;

MAG = [mp0p1;mp1p2;mp2p3;mp3p4];
ttotal = 0;

                           % ACCELERATION VECTOR
% MOTION POINTS P0 to P4
for i = 1: NP
    t1(i)=sqrt(2*MAG(i)*pad/accel);
    vf(i)=sqrt(2*accel*pad*MAG(i));
    t2(i)=(1-2*pad)*MAG(i)/vf(i);
    ta=[(-vf(i)+sqrt(vf(i)^2-4*0.5*-accel*-pad*MAG(i)))/(2*0.5*-accel);(-vf(i)-sqrt(vf(i)^2-4*0.5*-accel*-pad*MAG(i)))/(2*0.5*-accel)];
    if(ta(1,1)>0)
        t3(i)=real(ta(1,1));
    else
        t3(i)=real(ta(2,1));
    end
    ttotal = ttotal + t1(i)+t2(i)+t3(i);
    spd(i) = vf(i);
end

VTIME = linspace(0,ttotal,ESP)';
int = [t1;t2;t3];

% TIME DISTRIBUTION
INT(1) = int(1);
for i = 2: 3*NP
    INT(i) = INT(i-1)+int(i);
end

for i = 1: ESP
    if(VTIME(i) <= INT(1) || (VTIME(i) >= INT(3) && VTIME(i) <= INT(4)) || ...
      (VTIME(i) >= INT(6) && VTIME(i) <= INT(7)) || (VTIME(i) >= INT(9) && VTIME(i) <= INT(10)))
        AC(i) = accel;
    elseif((VTIME(i) >= INT(2) && (VTIME(i) < INT(3))) || (VTIME(i) >= INT(5) && VTIME(i) < INT(6)) || ...
           (VTIME(i) >= INT(8) && VTIME(i) < INT(9)) || (VTIME(i) >= INT(11) && VTIME(i) <= INT(12)))
        AC(i) = -accel;
    else
        AC(i) = 0;
    end
end

%% LOOP FOR SIMULATION MOTION
for i = 1: ESP
        %P0 - P1
    if(VTIME(i) <= INT(1))
        vpp = up0p1*0.5*AC(i)*VTIME(i)^2;
        mov(i) = 0.5*AC(i)*VTIME(i)^2;
        vel(i) = AC(i)*VTIME(i);
        vo = vel(i);
    elseif(VTIME(i) > INT(1) && VTIME(i) < INT(2))
        vpp = up0p1*(spd(1)*(VTIME(i)-INT(1))+0.5*accel*int(1)^2);
        mov(i) = (spd(1)*(VTIME(i)-INT(1))+0.5*accel*int(1)^2);
        vel(i) = AC(i)*(VTIME(i)-INT(1)) + vo;        
    elseif(VTIME(i) >= INT(2) && VTIME(i) < INT(3))
        vpp = up0p1*(0.5*AC(i)*(VTIME(i)-INT(2))^2+spd(1)*(VTIME(i)-INT(2))+0.5*accel*int(1)^2+spd(1)*int(2));
        mov(i) = (0.5*AC(i)*(VTIME(i)-INT(2))^2+spd(1)*(VTIME(i)-INT(2))+0.5*accel*int(1)^2+spd(1)*int(2));
        vel(i) = AC(i)*(VTIME(i)-INT(2))+vo;
        
        %P1 - P2
    elseif(VTIME(i) >= INT(3) && VTIME(i) <= INT(4))
        vpp = up1p2*0.5*AC(i)*(VTIME(i)-INT(3))^2;
        mov(i) = 0.5*AC(i)*(VTIME(i)-INT(3))^2;
        vel(i) = AC(i)*(VTIME(i)-INT(3));
        vo = vel(i);
    elseif(VTIME(i) > INT(4) && VTIME(i) < INT(5))
        vpp = up1p2*(spd(2)*(VTIME(i)-INT(4))+0.5*accel*int(4)^2);
        mov(i) = (spd(2)*(VTIME(i)-INT(4))+0.5*accel*int(4)^2);
        vel(i) = AC(i)*(VTIME(i)-INT(4))+vo; 
    elseif(VTIME(i) >= INT(5) && VTIME(i) < INT(6))
        vpp = up1p2*(0.5*AC(i)*(VTIME(i)-INT(5))^2+spd(2)*(VTIME(i)-INT(5))+0.5*accel*int(4)^2+spd(2)*int(5));
        mov(i) = (0.5*AC(i)*(VTIME(i)-INT(5))^2+spd(2)*(VTIME(i)-INT(5))+0.5*accel*int(4)^2+spd(2)*int(5));
        vel(i) = AC(i)*(VTIME(i)-INT(5))+vo;
    
        %P2 - P3
    elseif(VTIME(i) >= INT(6) && VTIME(i) <= INT(7))
        vpp = up2p3*0.5*AC(i)*(VTIME(i)-INT(6))^2;
        mov(i) = 0.5*AC(i)*(VTIME(i)-INT(6))^2;
        vel(i) = AC(i)*(VTIME(i)-INT(6));
        vo = vel(i);
    elseif(VTIME(i) > INT(7) && VTIME(i) < INT(8))
        vpp = up2p3*(spd(3)*(VTIME(i)-INT(7))+0.5*accel*int(7)^2);
        mov(i) = (spd(3)*(VTIME(i)-INT(7))+0.5*accel*int(7)^2);
        vel(i) = AC(i)*(VTIME(i)-INT(7))+vo; 
    elseif(VTIME(i) >= INT(8) && VTIME(i) < INT(9))
        vpp = up2p3*(0.5*AC(i)*(VTIME(i)-INT(8))^2+spd(3)*(VTIME(i)-INT(8))+0.5*accel*int(7)^2+spd(3)*int(8));
        mov(i) = (0.5*AC(i)*(VTIME(i)-INT(8))^2+spd(3)*(VTIME(i)-INT(8))+0.5*accel*int(7)^2+spd(3)*int(8));
        vel(i) = AC(i)*(VTIME(i)-INT(8))+vo;
     
        %P3 - P4
    elseif(VTIME(i) >= INT(9) && VTIME(i) <= INT(10))
        vpp = up3p4*0.5*AC(i)*(VTIME(i)-INT(9))^2;
        mov(i) = 0.5*AC(i)*(VTIME(i)-INT(9))^2;
        vel(i) = AC(i)*(VTIME(i)-INT(9));
        vo = vel(i);
    elseif(VTIME(i) > INT(10) && VTIME(i) < INT(11))
        vpp = up3p4*(spd(4)*(VTIME(i)-INT(10))+0.5*accel*int(10)^2);
        mov(i) = (spd(4)*(VTIME(i)-INT(10))+0.5*accel*int(10)^2);
        vel(i) = AC(i)*(VTIME(i)-INT(10))+vo; 
    elseif(VTIME(i) >= INT(11))
        vpp = up3p4*(0.5*AC(i)*(VTIME(i)-INT(11))^2+spd(4)*(VTIME(i)-INT(11))+0.5*accel*int(10)^2+spd(4)*int(11));
        mov(i) = (0.5*AC(i)*(VTIME(i)-INT(11))^2+spd(4)*(VTIME(i)-INT(11))+0.5*accel*int(10)^2+spd(4)*int(11));
        vel(i) = AC(i)*(VTIME(i)-INT(11))+vo;
    end

    % CONDITIONS OF THE LINES  P0-P1   P1-P2   P2-P3   P3-P4
    if(VTIME(i) >= 0 && VTIME(i) < INT(3))
        PM = Pti - vpp;
    elseif(VTIME(i) >= INT(3) && VTIME(i) < INT(6))
        PM = P1 - vpp;
    elseif(VTIME(i) >= INT(6) && VTIME(i) < INT(9))
        PM = P2 - vpp;
    elseif(VTIME(i) >= INT(9))
        PM = P3 - vpp;
    end
    % BACK UP POSITION VECTOR
    Buvx(i) = PM(1,1);
    Buvy(i) = PM(2,1);
    Buvz(i) = PM(3,1);
     %% HANDLER ORIENTATION
     rx = operrx;
     ry = operry;
     tpx = PM(1,1);
     tpy = PM(2,1);
     tpz = PM(3,1);
     
     RPx = [1 0 0 0;0 cosd(rx) -sind(rx) 0;0 sind(rx) cosd(rx) 0;0 0 0 1];
     RPy = [cosd(ry) 0 sind(ry) 0;0 1 0 0;-sind(ry) 0 cosd(ry) 0;0 0 0 1];
     TPxy = [1 0 0 tpx; 0 1 0 tpy; 0 0 1 tpz;0 0 0 1];
     P = O*TPxy*RPx*RPy;
     PP = P((1:3),4);
     Q = P*Tzn;
     PQ = Q((1:3),4);
     %% SHAPE POINTS 3D
     H = Q*THxy;
     PH = H((1:3),4);
     I = Q*TIxy;
     PI = I((1:3),4);
     J = Q*TJxy;
     PJ = J((1:3),4);
     %% ITERATION PART
     N = 4000;
     adv = linspace(0,l3,N);
     
     counter = 1;
     point = 1;
     PW = [PH,PI,PJ];
     uW = [uBE,uCF,uDG];
     number = [two,three,four];
     while point <= 3
         for j=1:N
             AV = uW(:,point)*adv(j);
             POS = number(:,point)-AV;
             SIGMA = norm(POS-PW(:,point));
             ERROR = abs(SIGMA-bar);
             if ERROR <= 0.1
                 PS(1,point) = POS(1,1);
                 PS(2,point) = POS(2,1);
                 PS(3,point) = POS(3,1);
                 VL(point) = SIGMA;
                 vrW(point) = norm(eight-PW(:,point));
                 counter = 1;
                 break;
             end
             counter = counter + 1;
         end
         
         if counter >= N || vrW(point) <= Rm
             disp(' ');disp(['LA POSICION ',num2str(point),': NO EXISTE O MECANICAMENTE ES IMPOSIBLE']);disp(' ');
             break;
         end
         point = point + 1;
     end
     % BACK UP OF SLIDER POSITION
     SP1(:,i) = PS(:,1);
     SP2(:,i) = PS(:,2);
     SP3(:,i) = PS(:,3);
     
     %% GRAPHICS
     % MAIN COORDINATE SYSTEM
     clf
     comp = figure(1);
     taman =[20,20,600,600];
     set(comp,'Position',taman);
     frame(O,'r',50);grid off; axis([-250 250 -250 250 0 500]);view([60 40 60]);hold on;
     % REPRESENTATIVE POINTS
     text(one(1,1)+20,one(2,1),one(3,1),'A','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(two(1,1)+20,two(2,1)-20,two(3,1),'B','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(three(1,1)+20,three(2,1)+20,three(3,1),'C','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(four(1,1)+20,four(2,1)-20,four(3,1),'D','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(five(1,1)+20,five(2,1)-20,five(3,1),'E','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(six(1,1)+20,six(2,1)+20,six(3,1),'F','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     text(seven(1,1)-20,seven(2,1)-20,seven(3,1),'G','BackgroundColor',[.9 .9 .1],'FontWeight','bold','FontSize',8);
     
     plot3([one(1,1) two(1,1)],[one(2,1) two(2,1)],[one(3,1) two(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) three(1,1)],[one(2,1) three(2,1)],[one(3,1) three(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) four(1,1)],[one(2,1) four(2,1)],[one(3,1) four(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([two(1,1) five(1,1)],[two(2,1) five(2,1)],[two(3,1) five(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([three(1,1) six(1,1)],[three(2,1) six(2,1)],[three(3,1) six(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([four(1,1) seven(1,1)],[four(2,1) seven(2,1)],[four(3,1) seven(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     plot3([one(1,1) eight(1,1)],[one(2,1) eight(2,1)],[one(3,1) eight(3,1)],'Color',[.53 .54 .56],'linewidth',2);
     
     sx=[five(1,1) six(1,1) seven(1,1)]';
     sy=[five(2,1) six(2,1) seven(2,1)]';
     sz=[five(3,1) six(3,1) seven(3,1)]';
     patch(sx,sy,sz,[.53 .54 .56]);
     % MAIN SHAPE AND HANDLER TOOL
     SX=[PH(1,1) PI(1,1) PJ(1,1)]';
     SY=[PH(2,1) PI(2,1) PJ(2,1)]';
     SZ=[PH(3,1) PI(3,1) PJ(3,1)]';
     patch(SX,SY,SZ,[.65 .8 .2]);
     plot3([PP(1,1) PQ(1,1)],[PP(2,1) PQ(2,1)],[PP(3,1) PQ(3,1)],'Color',[.0 .0 .8],'linewidth',2);
     
     % TEST POSITION
     plot3(PS(1,1),PS(2,1),PS(3,1),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8);
     plot3(PS(1,2),PS(2,2),PS(3,2),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
     plot3(PS(1,3),PS(2,3),PS(3,3),'s','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
     
     plot3([PS(1,1) PH(1,1)],[PS(2,1) PH(2,1)],[PS(3,1) PH(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     plot3([PS(1,2) PI(1,1)],[PS(2,2) PI(2,1)],[PS(3,2) PI(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     plot3([PS(1,3) PJ(1,1)],[PS(2,3) PJ(2,1)],[PS(3,3) PJ(3,1)],'Color',[.8 .6 .2],'linewidth',2);
     % PATH OF THE TRAJECTORY
     for k = 1:i
         plot3(Buvx(k),Buvy(k),Buvz(k),'.m','MarkerSize',4);
     end
     axis square
     pause (0.01)
end
mtime = ceil(ttotal);
divtime = 5;
mnmov = floor(min(mov)); mxmov = ceil(max(mov)); divmov = ceil(mxmov/12);
mnvel = floor(min(vel)); mxvel = ceil(max(vel)); divvel = ceil(mxvel/12);
mnacc = floor(min(AC)); mxacc = ceil(max(AC)); divacc = ceil(mxacc/12);

figure(2)
plot(VTIME,mov,'r','Linewidth',2);grid on;
xlabel('Time [s]');set(gca,'XTick',0:divtime:mtime);ylabel('Position [mm]');set(gca,'YTick',0:divmov:mxmov,'Ylim',[mnmov mxmov]);title('Position Vs Time');
figure(3)
plot(VTIME,vel,'g','Linewidth',2);grid on;    
xlabel('Time [s]');set(gca,'XTick',0:divtime:mtime);ylabel('Velocity [mm/s]');set(gca,'YTick',-1:divvel:mxvel,'Ylim',[mnvel mxvel]);title('Velocity Vs Time');
figure(4)
plot(VTIME,AC,'Linewidth',2);grid on;
xlabel('Time [s]');set(gca,'XTick',0:divtime:mtime);ylabel('Acceleration [mm/s^2]');set(gca,'YTick',mnacc:divacc:mxacc,'Ylim',[mnacc mxacc]);title('Acceleration Vs Time');
%% SLIDER VELOCITY ANALISYS
vdif1(1) = 0;
vdif2(1) = 0;
vdif3(1) = 0;
for i = 2:ESP
    vdif1(i) = norm(SP1(:,i)-SP1(:,i-1));
    vdif2(i) = norm(SP2(:,i)-SP2(:,i-1));    
    vdif3(i) = norm(SP3(:,i)-SP3(:,i-1));    
end
figure(5)
plot(VTIME,vdif1,VTIME,vdif2,VTIME,vdif3);
xlabel('Time [s]');ylabel('Differential Displacement');title('Differential Displacement Vs Time')
legend('Diff_SP1','Diff_SP2','Diff_SP3');grid on;
%% POSITION OF THE SLIDER, BASED ON THE REFERENCE SYSTEM
for i = 1:ESP
    nSP1(i) = norm(SP1(:,i));
    nSP2(i) = norm(SP2(:,i));
    nSP3(i) = norm(SP3(:,i));
end
vmaxnSP = max([max(nSP1);max(nSP2);max(nSP3)]);
nSP1 = nSP1./vmaxnSP;
nSP2 = nSP2./vmaxnSP;
nSP3 = nSP3./vmaxnSP;
figure(6)
plot(VTIME,nSP1,VTIME,nSP2,VTIME,nSP3);
xlabel('Time [s]');ylabel('Unit vector');title('Unit Vector Vs Time')
legend('Vu_SP1','Vu_SP2','Vu_SP3');grid on;
